"""
PYTEST_DONT_REWRITE
"""
from .app import Application, show_routes

from .build import main
from .build.tasks import Task

from .bases.service import Service
from .bases.hooks import Hook, Return
from .bases.components import Component
from .bases.controller import Controller
from .bases.response import FileResponse
from .bases.model_factory import ModelFactory
from .bases.entities import Session, Cookie, FormParam, FileStream, inject

from .console import main as console

from .solo import Solo
from .solo.manager import SoloManager

from .types import Type, AsyncType, TypeEncoder, validators

from .route import route, get, post, delete, put, options, websocket

from .helper import redirect, require

__version__ = "1.0.10"    



import os
import sys
import asyncio
import inspect

from IPython import embed
from IPython.core import formatters
from toolkit import cache_property

from .mocker import Mocker
from ..bases.manager import Manager
from ..helper import get_real_method

# bugfix
formatters.get_real_method = get_real_method


class ConsoleManager(Manager):

    def __init__(self):
        self.initialize(os.getcwd())
        self.finalize()
        self.mock_keys = list()

    async def resolve(self, type):
        def wrapper(arg: type):
            return arg

        return await self.injector.run_async([wrapper], dict(self.state))

    def mock(self, mocker):
        """
        构建测试数据
        :param mocker: Mocker对象
        :return:
        """
        for type, val, param_name in mocker:
            param_name = param_name or type.__class__.__name__.lower()
            parameter = inspect.Parameter(
                param_name, inspect._POSITIONAL_OR_KEYWORD, annotation=type)
            for component in self.injector.components:
                if component.can_handle_parameter(parameter):
                    identity = component.identity(parameter)
                    break
            else:
                raise RuntimeError(f"Type: {type} cannot be mocked! ")

            self.mock_keys.append(identity)
            self.state[identity] = val
            self.injector.initial[identity] = type

    def __getattr__(self, item):
        assert item in self.beans, f"{item} cannot inject!"
        beans = self.beans[item]
        if len(beans) == 1:
            bean, _ = beans[0]
        else:
            i = input("Same bean name: {}: ".format(
                ", ".join(f"({index+1}) of {module}" for index, (_, module)
                                                in enumerate(beans))))
            bean = beans[int(i) - 1][0]
        return self.await(self.resolve(bean))

    def __getitem__(self, item):
        return self.__getattr__(item)

    @cache_property
    def beans(self):
        beans = dict()
        for component in self.components:
            type = component.resolve.__annotations__["return"]
            beans.setdefault(type.__name__, []).append((type, type.__module__))
        return beans

    @staticmethod
    def _await(awaitable):
        """
        模拟await关键字
        :param awaitable:
        :return:
        """
        loop = asyncio.get_event_loop()
        task = loop.create_task(awaitable.__await__())
        loop.run_until_complete(task)
        return task.result()

    def clear(self):
        """
        清除mock数据
        :return:
        """
        for key in self.mock_keys:
            del self.state[key]
            del self.injector.initial[key]

    def start(self):
        if sys.version_info < (3, 7, 0):
            await = self._await
        else:
            _await = self._await
        mock = self.mock

        def inject(class_name):
            return getattr(self, class_name)

        embed()


def main():
    ConsoleManager().start()



# -*- coding:utf-8 -*-
import os
import re


from setuptools import setup, find_packages


def get_version(package):
    """
    Return package version as listed in `__version__` in `__init__.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    mth = re.search("__version__\s?=\s?['\"]([^'\"]+)['\"]", init_py)
    if mth:
        return mth.group(1)
    else:
        raise RuntimeError("Cannot find version!")


def install_requires():
    """
    Return requires in requirements.txt
    :return:
    """
    try:
        with open("requirements.txt") as f:
            return [line.strip() for line in f.readlines() if line.strip()]
    except OSError:
        return []

try:
    LONG_DESCRIPTION = open("README.rst").read()
except UnicodeDecodeError:
    LONG_DESCRIPTION = open("README.rst", encoding="utf-8").read()

setup(
    name="apistellar",
    version=get_version("apistellar"),
    description="enhance apistar web framework. ",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Intended Audience :: Developers",
        "Operating System :: Unix",
    ],
    keywords="apistar",
    author="cn",
    author_email="cnaafhvk@foxmail.com",
    url="https://www.github.com/ShichaoMa/apistellar",
    entry_points={
        "console_scripts": [
            "apistar-create = apistellar:main",
            "apistar-console = apistellar:console",
            "apistar-routes = apistellar:show_routes"
        ]
    },
    license="MIT",
    packages=find_packages(exclude=("tests*",)),
    install_requires=install_requires(),
    include_package_data=True,
    zip_safe=True,
    setup_requires=["pytest-runner"],
    tests_require=["pytest-apistellar", "pytest-asyncio", "pytest-cov"]
)
